package com.fjhx.xmhjc.service.column.impl;

import cn.hutool.core.util.StrUtil;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.xmhjc.entity.about.po.AboutUsCulture;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSelectDto;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubSelectDto;
import com.fjhx.xmhjc.entity.column.po.ColumnArticle;
import com.fjhx.xmhjc.entity.column.po.ColumnMenu;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuSubVo;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuVo;
import com.fjhx.xmhjc.mapper.column.ColumnArticleMapper;
import com.fjhx.xmhjc.service.column.ColumnArticleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.xmhjc.service.column.ColumnMenuService;
import com.fjhx.xmhjc.service.column.ColumnMenuSubService;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.column.vo.ColumnArticleVo;
import com.fjhx.xmhjc.entity.column.dto.ColumnArticleSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.column.dto.ColumnArticleDto;
import cn.hutool.core.bean.BeanUtil;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 栏目文章表 服务实现类
 * </p>
 *
 * @author lqh
 * @since 2023-11-16
 */
@Service
public class ColumnArticleServiceImpl extends ServiceImpl<ColumnArticleMapper, ColumnArticle> implements ColumnArticleService {

    @Resource
    private ColumnMenuService columnMenuService;
    @Resource
    private ColumnMenuSubService columnMenuSubService;

    @Override
    public List<ColumnArticleVo> getList(ColumnArticleSelectDto dto) {
        IWrapper<ColumnArticle> wrapper = getWrapper();
        wrapper.keyword(dto, new SqlField("ca", ColumnArticle::getSort));
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("ca", ColumnArticle::getStatus, dto.getStatus());
        }
        if(Objects.nonNull(dto.getColumnId())){
            wrapper.eq("ca", ColumnArticle::getColumnId, dto.getColumnId());
        }
        if(Objects.nonNull(dto.getSubId())){
            wrapper.eq("ca", ColumnArticle::getSubId, dto.getSubId());
        }
        if(StrUtil.isNotBlank(dto.getTitle())){
            wrapper.like("ca", ColumnArticle::getTitle, dto.getTitle());
        }
        wrapper.orderByAsc("ca", ColumnArticle::getSort);
        List<ColumnArticleVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ColumnArticleVo> getPage(ColumnArticleSelectDto dto) {
        IWrapper<ColumnArticle> wrapper = getWrapper();
        wrapper.keyword(dto, new SqlField("ca", ColumnArticle::getTitle));
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("ca", ColumnArticle::getStatus, dto.getStatus());
        }
        if(Objects.nonNull(dto.getColumnId())){
            wrapper.eq("ca", ColumnArticle::getColumnId, dto.getColumnId());
        }
        if(Objects.nonNull(dto.getSubId())){
            wrapper.eq("ca", ColumnArticle::getSubId, dto.getSubId());
        }
        wrapper.orderByAsc("ca", ColumnArticle::getSort);
        Page<ColumnArticleVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        Map<Long, String> map = columnMenuService.getList(new ColumnMenuSelectDto()).stream().collect(Collectors.toMap(ColumnMenuVo::getId, ColumnMenuVo::getName));
        Map<Long, String> map1 = columnMenuSubService.getList(new ColumnMenuSubSelectDto()).stream().collect(Collectors.toMap(ColumnMenuSubVo::getId, ColumnMenuSubVo::getName));

        page.getRecords().forEach(article ->{
            article.setColumnIdName(map.getOrDefault(article.getColumnId(),""));
            article.setSubIdName(map1.getOrDefault(article.getSubId(),""));
        } );

        return page;
    }

    @Override
    public ColumnArticleVo detail(Long id) {
        ColumnArticle ColumnArticle = this.getById(id);
        ColumnArticleVo result = BeanUtil.toBean(ColumnArticle, ColumnArticleVo.class);
        return result;
    }

    @Override
    public void add(ColumnArticleDto columnArticleDto) {
        this.save(columnArticleDto);

        // 封面
        ObsFileUtil.saveFile(columnArticleDto.getCoverList(), columnArticleDto.getId(), 1);
        ObsFileUtil.saveFile(columnArticleDto.getVideoList(), columnArticleDto.getId(), 2);
    }

    @Override
    public void edit(ColumnArticleDto columnArticleDto) {
        this.updateById(columnArticleDto);

        // 封面
        ObsFileUtil.editFile(columnArticleDto.getCoverList(), columnArticleDto.getId(), 1);
        ObsFileUtil.editFile(columnArticleDto.getVideoList(), columnArticleDto.getId(), 2);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Page<ColumnArticleVo> getArticleByOpen(ColumnArticleSelectDto dto) {
        IWrapper<ColumnArticle> wrapper = getWrapper();
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("ca", ColumnArticle::getStatus, dto.getStatus());
        }
        if(Objects.nonNull(dto.getColumnId())){
            wrapper.eq("ca", ColumnArticle::getColumnId, dto.getColumnId());
        }
        if(Objects.nonNull(dto.getSubId())){
            wrapper.eq("ca", ColumnArticle::getSubId, dto.getSubId());
        }
        wrapper.orderByAsc("ca", ColumnArticle::getSort);
        Page<ColumnArticleVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        page.getRecords().forEach(x->{
            //获取封面图片
            List<Long> businessIdList = new ArrayList<>();
            businessIdList.add(x.getId());
            Map<Long, List<FileInfoVo>> fileMap = ObsFileUtil.getFileMap(businessIdList, 1);
            x.setCoverList(fileMap.get(x.getId()));
        });
        return page;
    }

    @Override
    public ColumnArticleVo getArticleDetailByOpen(Long id) {
        ColumnArticle ColumnArticle = this.getById(id);
        ColumnArticleVo result = BeanUtil.toBean(ColumnArticle, ColumnArticleVo.class);
        return result;
    }

    @Override
    public void saveReadyArticleNum(Long id) {
        ColumnArticle columnArticle = this.getById(id);
        columnArticle.setReadNum(Math.addExact(columnArticle.getReadNum(),1L));
        updateById(columnArticle);
    }

    /**
     * 相关文章
     * 根据点击量排序
     * @param dto
     * @return
     */
    @Override
    public Page<ColumnArticleVo> relatedArticles(ColumnArticleSelectDto dto) {
        IWrapper<ColumnArticle> wrapper = getWrapper();
        wrapper.keyword(dto, new SqlField("ca", ColumnArticle::getTitle));
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("ca", ColumnArticle::getStatus, dto.getStatus());
        }
        if(Objects.nonNull(dto.getSubId())){
            wrapper.eq("ca", ColumnArticle::getSubId, dto.getSubId());
        }
        wrapper.orderByDesc("ca", ColumnArticle::getReadNum);
        Page<ColumnArticleVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }
}
