package com.fjhx.xmhjc.service.topic.impl;

import cn.hutool.core.util.ObjectUtil;
import com.fjhx.xmhjc.entity.topic.po.TopicReplies;
import com.fjhx.xmhjc.mapper.topic.TopicRepliesMapper;
import com.fjhx.xmhjc.service.topic.TopicRepliesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.topic.vo.TopicRepliesVo;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesDto;
import cn.hutool.core.bean.BeanUtil;

import java.util.List;

/**
 * <p>
 * 主题回复 服务实现类
 * </p>
 *
 * @author hj
 * @since 2023-11-20
 */
@Service
public class TopicRepliesServiceImpl extends ServiceImpl<TopicRepliesMapper, TopicReplies> implements TopicRepliesService {

    @Override
    public List<TopicRepliesVo> getList(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByDesc("tr", TopicReplies::getId);
        List<TopicRepliesVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<TopicRepliesVo> getPage(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByDesc("tr", TopicReplies::getId);
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public TopicRepliesVo detail(Long id) {
        TopicReplies TopicReplies = this.getById(id);
        TopicRepliesVo result = BeanUtil.toBean(TopicReplies, TopicRepliesVo.class);
        return result;
    }

    @Override
    public void add(TopicRepliesDto topicRepliesDto) {
        this.save(topicRepliesDto);
    }

    @Override
    public void edit(TopicRepliesDto topicRepliesDto) {
        this.updateById(topicRepliesDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Page<TopicRepliesVo> getPageByOpen(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByAsc("tr", TopicReplies::getId);
        wrapper.eq("tr", TopicReplies::getTopicId, dto.getTopicId());
        wrapper.eq(ObjectUtil.isNotNull(dto.getFloorId()), "floor_id", dto.getFloorId());
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public Page<TopicRepliesVo> replyPage(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByAsc("tr", TopicReplies::getId);
        wrapper.eq("tr", TopicReplies::getTopicId, dto.getTopicId());
        wrapper.eq(ObjectUtil.isNotNull(dto.getFloorId()), "floor_id", dto.getFloorId());
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public Boolean hasReply(Long id) {
        Boolean hasReply = false;
        Long count = lambdaQuery().eq(TopicReplies::getTopicId, id).count();
        if (count > 0) {
            hasReply = true;
        }
        return hasReply;
    }
}
